#if !defined(USBCON)
#error "Where's my USBCON?"
#endif
#if defined(USBCON)
void ciao() {

}
#endif

void setup() {
  // put your setup code here, to run once:

}

void loop() {
  // put your main code here, to run repeatedly:

}