
const int MY_FIRST_PIN = FIRST_PIN;
const int MY_SECOND_PIN = SECOND_PIN;

void setup() {
    Serial.begin(9600);
    Serial.println(MY_FIRST_PIN);
    Serial.println(MY_SECOND_PIN);
}

void loop() {
}
