// Arduino.h should not be automatically included by the Arduino
// preprocessor before the explicit include line in this sketch.

#if defined(HIGH)
#error Arduino.h seems to be automatically included
#endif

   #    include		<Arduino.h>

void setup() {
}

void loop() {
}
